import Crossover from "../types/Crossover";
import Fitness from "../types/Fitness";
import Mutate from "../types/Mutate";
import Seed from "../types/Seed";
import Stop from "../types/Stop";
export declare type SelectMethod = "fittest" | "random" | "tournament2" | "tournament3";
export declare type SelectFunction<T> = (creatures: T[]) => T;
export declare type Optimize = "minimize" | "maximize";
export declare type Creature = {
    fitness: number;
};
export declare type SupportMethods = Map<string, (args: any[]) => any>;
export declare type GeneticConfig = {
    selectElite: number;
    selectEliteMethod: SelectMethod;
    optimize: Optimize;
    populationSize: number;
    crossoverRate: number;
    crossoverMethod: SelectMethod;
    mutationRate: number;
    iterations: number;
    isWorker: boolean;
};
declare class GA<M, D, T extends Creature> {
    randInt(min: number, max: number): number;
    readonly seed: Seed<GA<M, D, T>, T>;
    readonly fit: Fitness<GA<M, D, T>, T>;
    readonly mutate: Mutate<GA<M, D, T>, T>;
    readonly crossover: Crossover<GA<M, D, T>, T>;
    readonly stop: Stop<GA<M, D, T>, T[]>;
    readonly methods: M;
    readonly config: GeneticConfig;
    data: D;
    population: T[];
    constructor(seed: Seed<GA<M, D, T>, T>, fit: Fitness<GA<M, D, T>, T>, mutate: Mutate<GA<M, D, T>, T>, crossover: Crossover<GA<M, D, T>, T>, stop: Stop<GA<M, D, T>, T[]>, methods: M, config: GeneticConfig);
    solve: (data: D) => Promise<T[]>;
    private __solve;
}
export default GA;
