"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const unified_worker_1 = require("unified-worker");
class GA {
    constructor(seed, fit, mutate, crossover, stop, methods, config) {
        this.solve = (data) => __awaiter(this, void 0, void 0, function* () {
            this.data = data;
            this.population = [];
            if (this.config.isWorker)
                this.population = yield (0, unified_worker_1.worker)(this.__solve, this, []);
            else
                this.population = this.__solve();
            return this.population;
        });
        this.__solve = () => {
            const evolve = () => {
                this.population.forEach((creature) => (creature.fitness = this.fit(creature)));
                return mutate(crossover(this.population));
            };
            const mutate = (creatures) => {
                for (let i = this.config.selectElite; i < this.config.populationSize; i++)
                    creatures[i] = this.mutate(creatures[i]);
                return creatures;
            };
            const crossover = (creatures) => {
                const methods = new Map([
                    ["fittest", fittestSelection.bind(this)],
                    ["random", randomSelection.bind(this)],
                    ["tournament2", tournament2Selection.bind(this)],
                    ["tournament3", tournament3Selection.bind(this)],
                ]);
                const elite = [];
                for (let i = 0; i < this.config.selectElite; i++)
                    elite.push(methods.get(this.config.selectEliteMethod)(creatures));
                const crossed = [];
                for (let i = this.config.selectElite; i < this.config.populationSize; i++) {
                    const mother = methods.get(this.config.crossoverMethod)(creatures);
                    const father = methods.get(this.config.crossoverMethod)(creatures);
                    crossed.push(this.crossover(mother, father));
                }
                return [...elite, ...crossed];
            };
            const fittestSelection = (creatures) => creatures.sort((c1, c2) => c2.fitness - c1.fitness)[0];
            const randomSelection = (creatures) => choice(creatures);
            const tournament2Selection = (creatures) => tournament(creatures, 2);
            const tournament3Selection = (creatures) => tournament(creatures, 3);
            const tournament = (creatures, size) => {
                const creaturesToCompare = [];
                for (let i = 0; i < size; i++)
                    creaturesToCompare.push(choice(creatures));
                return creaturesToCompare.sort(sort.bind(this))[0];
            };
            function sort(c1, c2) {
                if (this.config.optimize === "minimize")
                    return c1.fitness - c2.fitness;
                else
                    return c2.fitness - c1.fitness;
            }
            const choice = (array) => array[Math.floor(Math.random() * array.length)];
            for (let i = 0; i < this.config.populationSize; i++)
                this.population.push(this.seed());
            let generationCount = 0;
            while (generationCount < this.config.iterations && !this.stop(this.population)) {
                this.population = evolve();
            }
            return this.population;
        };
        this.seed = seed;
        this.fit = fit;
        this.mutate = mutate;
        this.crossover = crossover;
        this.stop = stop;
        this.methods = methods;
        this.config = config;
    }
    randInt(min, max) {
        throw new Error("Method not implemented.");
    }
}
exports.default = GA;
//# sourceMappingURL=GA.js.map